@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module YARD
   # The root path for YARD source libraries
   ROOT = File.expand_path(File.dirname(__FILE__))
@@ -33,11 +34,8 @@ module YARD
   def self.windows?
     return @windows if defined? @windows
     require 'rbconfig'
-    if ::RbConfig::CONFIG['host_os'] =~ /mingw|win32|cygwin/
-      @windows = true
-    else
-      @windows = false
-    end
+    @windows =
+      ::RbConfig::CONFIG['host_os'] =~ /mingw|win32|cygwin/ ? true : false
   ensure
     @windows ||= false
   end
@@ -50,11 +48,15 @@ module YARD
 
   # @return [Boolean] whether YARD is being run in Ruby 2.0
   def self.ruby2?; @ruby2 ||= (RUBY_VERSION >= '2.0.0') end
+
+  # @return [Boolean] whether YARD is being run in Ruby 3.0
+  def self.ruby3?; @ruby3 ||= (RUBY_VERSION >= '3.0.0') end
 end
 
 # Keep track of Ruby version for compatibility code
 # @deprecated Use {YARD.ruby18?} or {YARD.ruby19?} instead.
-RUBY18, RUBY19 = YARD.ruby18?, YARD.ruby19?
+RUBY18 = YARD.ruby18?
+RUBY19 = YARD.ruby19?
 
 # Load Ruby core extension classes
 Dir.glob(File.join(YARD::ROOT, 'yard', 'core_ext', '*.rb')).each do |file|
