@@ -117,7 +117,7 @@ module YARD
     #
     #   --tag overload:"Overloaded Method"
     #
-    # See +yardoc --help+ for a description of the various options.
+    # See +yard help doc+ for a description of the various options.
     #
     # Tags added in this way are automatically displayed in output. To add
     # a meta-data tag that does not show up in output, use +--hide-tag TAG+.
@@ -200,6 +200,9 @@ module YARD
       # @since 0.7.0
       attr_accessor :has_markup
 
+      # @return [Boolean] whether yard exits with error status code if a warning occurs
+      attr_accessor :fail_on_warning
+
       # Creates a new instance of the commandline utility
       def initialize
         super
@@ -218,6 +221,7 @@ module YARD
         @list = false
         @save_yardoc = true
         @has_markup = false
+        @fail_on_warning = false
 
         if defined?(::Encoding) && ::Encoding.respond_to?(:default_external=)
           utf8 = ::Encoding.find('utf-8')
@@ -273,6 +277,8 @@ module YARD
           end
         end
 
+        abort if fail_on_warning && log.warned
+
         true
       ensure
         log.show_progress = false
@@ -288,9 +294,11 @@ module YARD
         # Last minute modifications
         self.files = Parser::SourceParser::DEFAULT_PATH_GLOB if files.empty?
         files.delete_if {|x| x =~ /\A\s*\Z/ } # remove empty ones
-        readme = Dir.glob('README{,*[^~]}').first
-        readme ||= Dir.glob(files.first).first if options.onefile
-        options.readme ||= CodeObjects::ExtraFileObject.new(readme) if readme
+        readme = Dir.glob('README{,*[^~]}').
+          select {|f| extra_file_valid?(f)}.
+          sort_by {|r| [r.count('.'), r.index('.'), r] }.first
+        readme ||= Dir.glob(files.first).first if options.onefile && !files.empty?
+        options.readme ||= CodeObjects::ExtraFileObject.new(readme) if readme && extra_file_valid?(readme)
         options.files.unshift(options.readme).uniq! if options.readme
 
         Tags::Library.visible_tags -= hidden_tags
@@ -405,14 +413,27 @@ module YARD
       def add_extra_files(*files)
         files.map! {|f| f.include?("*") ? Dir.glob(f) : f }.flatten!
         files.each do |file|
-          if File.file?(file)
+          if extra_file_valid?(file)
             options.files << CodeObjects::ExtraFileObject.new(file)
-          else
-            log.warn "Could not find extra file: #{file}"
           end
         end
       end
 
+      # @param file [String] the filename to validate
+      # @param check_exists [Boolean] whether the file should exist on disk
+      # @return [Boolean] whether the file is allowed to be used
+      def extra_file_valid?(file, check_exists = true)
+        if file =~ %r{^(?:\.\./|/)}
+          log.warn "Invalid file: #{file}"
+          false
+        elsif check_exists && !File.file?(file)
+          log.warn "Could not find file: #{file}"
+          false
+        else
+          true
+        end
+      end
+
       # Parses the file arguments into Ruby files and extra files, which are
       # separated by a '-' element.
       #
@@ -497,7 +518,7 @@ module YARD
         opts.banner = "Usage: yard doc [options] [source_files [- extra_files]]"
 
         opts.separator "(if a list of source files is omitted, "
-        opts.separator "  {lib,app}/**/*.rb ext/**/*.c is used.)"
+        opts.separator "  {lib,app}/**/*.rb ext/**/*.{c,rb} is used.)"
         opts.separator ""
         opts.separator "Example: yardoc -o documentation/ - FAQ LICENSE"
         opts.separator "  The above example outputs documentation for files in"
@@ -555,6 +576,10 @@ module YARD
         opts.on('--exclude REGEXP', 'Ignores a file if it matches path match (regexp)') do |path|
           excluded << path
         end
+
+        opts.on('--fail-on-warning', 'Exit with error status code if a warning occurs') do
+          self.fail_on_warning = true
+        end
       end
 
       # Adds output options
@@ -634,10 +659,8 @@ module YARD
 
         opts.on('-r', '--readme FILE', '--main FILE', 'The readme file used as the title page',
                                                       '  of documentation.') do |readme|
-          if File.file?(readme)
+          if extra_file_valid?(readme)
             options.readme = CodeObjects::ExtraFileObject.new(readme)
-          else
-            log.warn "Could not find readme file: #{readme}"
           end
         end
 
@@ -648,12 +671,9 @@ module YARD
 
         opts.on('--asset FROM[:TO]', 'A file or directory to copy over to output ',
                                      '  directory after generating') do |asset|
-          re = %r{^(?:\.\./|/)}
-          from, to = *asset.split(':').map {|f| File.cleanpath(f) }
+          from, to = *asset.split(':').map {|f| File.cleanpath(f, true) }
           to ||= from
-          if from =~ re || to =~ re
-            log.warn "Invalid file '#{asset}'"
-          else
+          if extra_file_valid?(from, false) && extra_file_valid?(to, false)
             assets[from] = to
           end
         end
