@@ -480,11 +480,22 @@ module YARD
         end
 
         def parameters(include_block_param = true)
-          params = self[1 + index_adjust]
+          return unless params = self[1 + index_adjust]
           params = params[0] if params.type == :paren
           include_block_param ? params : params[0...-1]
         end
 
+        def signature
+          params_src = ''
+          params = self[1 + index_adjust]
+          if params and params.first
+            params_src = params.type == :paren ? '' : ' '
+            params_src += params.source.gsub(/\s+(\s|\))/m, '\1')
+          end
+
+          "def #{method_name(true)}#{params_src}"
+        end
+
         alias block last
 
         private
