@@ -202,8 +202,7 @@ module YARD
             begin; undef on_#{event}; rescue NameError; end
             def on_#{event}(tok)
               unless @last_ns_token == [:kw, "def"] ||
-                  (@tokens.last && @tokens.last[0] == :symbeg) ||
-                  (!@newline && %w(if while until unless).include?(tok))
+                  (@tokens.last && @tokens.last[0] == :symbeg)
                 (@map[tok] ||= []) << [lineno, charno]
               end
               visit_ns_token(:#{event}, tok, true)
@@ -217,7 +216,7 @@ module YARD
             def on_#{event}(tok)
               add_token(:#{event}, tok)
               @newline = true
-              @charno += tok ? tok.length : 1
+              @charno += tok ? tok.length : 0
             end
           eof
         end
@@ -271,15 +270,20 @@ module YARD
               @percent_ary.source_range = rng
               @tokens << [token, data, [lineno, charno]]
               @percent_ary = nil
+              return
             elsif token == :tstring_end && data =~ /\A\s/
               rng = @percent_ary.source_range
               rng = Range.new(rng.first, rng.last + data.length)
               @percent_ary.source_range = rng
               @tokens << [token, data, [lineno, charno]]
               @percent_ary = nil
+              return
             end
-          elsif @tokens.last && @tokens.last[0] == :symbeg
-            @tokens[-1] = [:symbol, ":" + data, @tokens.last[2]]
+          end
+
+          if @tokens.last && (@tokens.last[0] == :symbeg ||
+              (@tokens.last[0] == :symbol && token.to_s =~ /^tstring/))
+            @tokens[-1] = [:symbol, @tokens.last[1] + data, @tokens.last[2]]
           elsif @heredoc_state == :started
             @heredoc_tokens << [token, data, [lineno, charno]]
 
@@ -328,6 +332,7 @@ module YARD
         undef on_bodystmt
         undef on_top_const_ref
         undef on_const_path_ref
+        undef on_dyna_symbol
 
         def on_program(*args)
           args.first
@@ -365,7 +370,7 @@ module YARD
 
         def on_aref(*args)
           @map[:lbracket].pop
-          ll, lc = *@map[:aref].pop
+          ll, lc = *@map[:aref].shift
           sr = args.first.source_range.first..lc
           lr = args.first.line_range.first..ll
           AstNode.new(:aref, args, :char => sr, :line => lr)
@@ -405,6 +410,16 @@ module YARD
           visit_ns_token(:rbracket, tok, false)
         end
 
+        def on_dyna_symbol(sym)
+          rng = if sym.source_range.to_a.size == 0 # rubocop:disable Style/ZeroLengthPredicate
+                  (sym.source_range.begin - 3)...sym.source_range.end
+                else
+                  (sym.source_range.begin - 2)..(sym.source_range.end + 1)
+                end
+          AstNode.new(:dyna_symbol, [sym], :line => lineno..lineno,
+            :listline => lineno..lineno, :listchar => rng, :char => rng)
+        end
+
         def on_top_const_ref(*args)
           type = :top_const_ref
           node = AstNode.node_class_for(type).new(type, args)
@@ -426,6 +441,8 @@ module YARD
           module_eval(<<-eof, __FILE__, __LINE__ + 1)
             begin; undef on_#{kw}; rescue NameError; end
             def on_#{kw}(*args)
+              mapping = @map[#{kw.to_s.sub(/_mod$/, '').inspect}]
+              mapping.pop if mapping
               sr = args.last.source_range.first..args.first.source_range.last
               lr = args.last.line_range.first..args.first.line_range.last
               #{node_class}.new(:#{kw}, args, :line => lr, :char => sr)
@@ -476,7 +493,8 @@ module YARD
         end
 
         def on_string_content(*args)
-          AstNode.new(:string_content, args, :listline => lineno..lineno, :listchar => charno..charno)
+          chr_rng = args.empty? ? charno...charno : charno..charno
+          AstNode.new(:string_content, args, :listline => lineno..lineno, :listchar => chr_rng)
         end
 
         def on_rescue(exc, *args)
@@ -595,7 +613,7 @@ module YARD
 
         def insert_comments
           root.traverse do |node|
-            next if node.type == :comment || node.type == :list || node.parent.type != :list
+            next if [:comment, :void_stmt, :list].include?(node.type) || node.parent.type != :list
 
             # never attach comments to if/unless mod nodes
             if node.type == :if_mod || node.type == :unless_mod
@@ -667,6 +685,7 @@ module YARD
         end
 
         def freeze_tree(node = nil)
+          @tokens = @tokens.sort_by {|t| t.last }
           nodes = [node || root]
           until nodes.empty?
             p_node = nodes.shift
